"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isBabelPluginFactory = _interopRequireDefault(require("../utils/is-babel-plugin-factory"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getReturnValue(node) {
  const {
    body
  } = node;

  if (body.type === "BlockStatement") {
    const returnNode = body.body.find(n => n.type === "ReturnStatement");
    return returnNode && returnNode.argument;
  }

  return body;
}

var _default = {
  meta: {
    schema: []
  },

  create(context) {
    let pluginFound = false;
    return {
      FunctionDeclaration: functionVisitor,
      FunctionExpression: functionVisitor,
      ArrowFunctionExpression: functionVisitor,

      "Program:exit"(node) {
        if (!pluginFound) {
          context.report(node, "This file does not export a Babel plugin.");
        }
      }

    };

    function functionVisitor(node) {
      if (!(0, _isBabelPluginFactory.default)(node, context.getScope())) return;
      const returnValue = getReturnValue(node);
      if (!returnValue || returnValue.type !== "ObjectExpression") return;
      pluginFound = true;

      if (!returnValue.properties.some(p => p.key.name === "name")) {
        context.report(returnValue, "This Babel plugin doesn't have a 'name' property.");
      }
    }
  }

};
exports.default = _default;